/*****************************************************************************
** STAIR VISION LIBRARY
** Copyright (c) 2007-2009, Stephen Gould
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**     * Redistributions of source code must retain the above copyright
**       notice, this list of conditions and the following disclaimer.
**     * Redistributions in binary form must reproduce the above copyright
**       notice, this list of conditions and the following disclaimer in the
**       documentation and/or other materials provided with the distribution.
**     * Neither the name of the Stanford University nor the
**       names of its contributors may be used to endorse or promote products
**       derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
** ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
******************************************************************************
** FILENAME:    testMachineLearning.cpp
** AUTHOR(S):   Stephen Gould <sgould@stanford.edu>
** DESCRIPTION:
**  Regression tests for machine learning classes.
**
*****************************************************************************/

#include <cstdlib>
#include <cassert>
#include <iostream>
#include <fstream>
#include <iomanip>
#include <cmath>

#include <cxcore.h>

#include "svlBase.h"
#include "svlML.h"
#include "svlVision.h"

using namespace std;

// prototypes ---------------------------------------------------------------

void testLogistic();
void testConditionalGaussian();

// main ---------------------------------------------------------------------

void usage()
{
    cerr << SVL_USAGE_HEADER << "\n"; 
    cerr << "USAGE: ./testMachineLearning [OPTIONS]\n";
    cerr << "OPTIONS:\n"
         << "  -logistic         :: test logistic regression\n"
         << "  -gaussian         :: test gaussians\n"
         << SVL_STANDARD_OPTIONS_USAGE
         << endl;
}

int main(int argc, char *argv[])
{
    if (argc <= 1) {
        usage();
        return -1;
    }

    char **args = argv + 1;
    while (--argc > 0) {
        SVL_PROCESS_STANDARD_OPTIONS(args, argc)
        else if (!strcmp(*args, "-logistic")) {
            testLogistic();
        } else if (!strcmp(*args, "-gaussian")) {
            testConditionalGaussian();
	} else {
	    cerr << "ERROR: unrecognized option " << *args << endl;
	    usage();
	    return -1;
	}
	args++;
    }

    svlCodeProfiler::print(cerr);
    return 0;
}

// tests ------------------------------------------------------------------

void testLogistic()
{
    SVL_ASSERT(false);
}

void testConditionalGaussian()
{
    CvMat *mu = cvCreateMat(3, 1, CV_32FC1);
    CvMat *sigma = cvCreateMat(3, 3, CV_32FC1);
    
    CV_MAT_ELEM(*mu, float, 0, 0) = 1.0;
    CV_MAT_ELEM(*mu, float, 1, 0) = 2.0;
    CV_MAT_ELEM(*mu, float, 2, 0) = 3.0;

    CV_MAT_ELEM(*sigma, float, 0, 0) = 1.0;
    CV_MAT_ELEM(*sigma, float, 0, 1) = 0.1;
    CV_MAT_ELEM(*sigma, float, 0, 2) = 0.2;
    CV_MAT_ELEM(*sigma, float, 1, 0) = CV_MAT_ELEM(*sigma, float, 0, 1);
    CV_MAT_ELEM(*sigma, float, 1, 1) = 1.0;
    CV_MAT_ELEM(*sigma, float, 1, 2) = -0.5;
    CV_MAT_ELEM(*sigma, float, 2, 0) = CV_MAT_ELEM(*sigma, float, 0, 2);
    CV_MAT_ELEM(*sigma, float, 2, 1) = CV_MAT_ELEM(*sigma, float, 1, 2);
    CV_MAT_ELEM(*sigma, float, 2, 2) = 1.0;

    svlGaussian gaussian(mu, sigma);

    cout << "Mean:\n" << toString(*gaussian.mean()) << endl;
    cout << "Variance:\n" << toString(*gaussian.covariance()) << endl;

    cvReleaseMat(&sigma);
    cvReleaseMat(&mu);

    vector<double> x(2);
    vector<int> indx(2);

    x[0] = 1.9; indx[0] = 1;
    x[1] = 3.2; indx[1] = 2;

    svlGaussian conditionalGaussian = gaussian.reduce(x, indx);

    cout << "Mean:\n" << toString(*conditionalGaussian.mean()) << endl;
    cout << "Variance:\n" << toString(*conditionalGaussian.covariance()) << endl;
}
