/******************************************************************************
** STAIR VISION LIBRARY
** Copyright (c) 2007-2009, Stephen Gould
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**     * Redistributions of source code must retain the above copyright
**       notice, this list of conditions and the following disclaimer.
**     * Redistributions in binary form must reproduce the above copyright
**       notice, this list of conditions and the following disclaimer in the
**       documentation and/or other materials provided with the distribution.
**     * Neither the name of the Stanford University nor the
**       names of its contributors may be used to endorse or promote products
**       derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
** ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
******************************************************************************
** FILENAME:    svlBox2d.h
** AUTHOR(S):   Paul Baumstarck <pbaumstarck@stanford.edu>
** DESCRIPTION:
**  Bounding box class for svlPoint2d's. 
*****************************************************************************/

#pragma once

#if defined(_WIN32)||defined(WIN32)||defined(__WIN32__)
#define _CRT_SECURE_NO_DEPRECATE
#undef max
#undef min
#endif

#include <limits>
#include <iostream>

#include "svlPoint2d.h"

using namespace std;

class svlBox2d {
public:
	svlPoint2d max_corner, min_corner;
public:
	inline svlBox2d()
	    : max_corner(svlPoint2d(-numeric_limits<double>::max())),
	    min_corner(svlPoint2d(numeric_limits<double>::max())) {}
    inline svlBox2d(const svlPoint2d& pt)
	    : max_corner(svlPoint2d(pt)),
	    min_corner(svlPoint2d(pt)) {}
	inline svlBox2d(const svlBox2d& box)
	    : max_corner(box.max_corner),
	    min_corner(box.min_corner) {}
	inline svlBox2d(const svlBox2d& box1, const svlBox2d& box2)
	    : max_corner(box1.max_corner),
	    min_corner(box1.min_corner) {
	    max_corner.max_with(box2.max_corner);
	    min_corner.min_with(box2.min_corner);
	}
	inline svlBox2d(const svlPoint2d& min_in, const svlPoint2d& max_in)
	    : max_corner(max_in), min_corner(min_in) {}
	virtual ~svlBox2d() {}; // do nothing

	inline void addToBox(const svlPoint2d &pt) {
		min_corner.min_with(pt);
		max_corner.max_with(pt);
	}

	inline void unionWithBox(const svlBox2d &box) {
		min_corner.min_with(box.min_corner);
		max_corner.max_with(box.max_corner);
	}

	inline double minDistFromPt(const svlPoint2d &pt) {
		return std::min( (max_corner-pt).pt_min(), (pt-min_corner).pt_min() );
	}

	inline double mDFPWOB(const svlPoint2d &pt, const svlBox2d &box) {
		return minDistFromPtWithOuterBox(pt,box);
	}

	inline double minDistFromPtWithOuterBox(const svlPoint2d &pt, const svlBox2d &box) {
		return std::min( svlPoint2d( max_corner.x == box.max_corner.x ? numeric_limits<double>::max() : max_corner.x - pt.x,
			max_corner.y == box.max_corner.y ? numeric_limits<double>::max() : max_corner.y - pt.y ).pt_min(),
		    svlPoint2d( min_corner.x == box.min_corner.x ? numeric_limits<double>::max() : pt.x - min_corner.x,
			min_corner.y == box.min_corner.y ? numeric_limits<double>::max() : pt.y - min_corner.y ).pt_min() );
	}

	friend inline svlBox2d operator/(const svlBox2d& p, double d) {
		return svlBox2d(p.min_corner, p.min_corner + (p.max_corner-p.min_corner)/2.0);
	}
	
	friend std::ostream& operator<<(std::ostream& os, const svlBox2d& p) {
	    os << "(" << p.min_corner << ") -> (" << p.max_corner << ")";
	    return os;
	}
};

