#include "svlDetectorFeatureExtractor.h"
#include "svlVision.h"

static const char* knownSubclasses[] = {
  "PatchDictionary",
  "CompositeFeatureExtractor",
  NULL
  };

//Helper method used by the two class members (one for string, one for file)
svlDetectorFeatureExtractor* getInstanceHelper(XMLNode &root, int subclassIdx);

svlDetectorFeatureExtractor*
  svlDetectorFeatureExtractor::getInstance(const string &filename)
{

  XMLNode root = XMLNode::parseFile(filename.c_str(), knownSubclasses[0]);

  //Try all possible root nodes until we find one that works
  int subclassIdx = 0;
  for(; root.isEmpty() && knownSubclasses[subclassIdx]!=NULL; subclassIdx++)
    {
      //cout << filename << " " << knownSubclasses[subclassIdx] << endl;
      root = XMLNode::parseFile(filename.c_str(), knownSubclasses[subclassIdx]);
    }

  if (root.isEmpty())
    {
      SVL_LOG(SVL_LOG_WARNING, "No valid xml root found");
    }

  return getInstanceHelper(root, subclassIdx);
}

svlDetectorFeatureExtractor*
svlDetectorFeatureExtractor::getInstanceFromString(const char *XML)
{
  XMLNode root = XMLNode::parseString(XML, knownSubclasses[0]);
  
  //Try all possible root nodes until we find one that works
  int subclassIdx = 1;
  for(; root.isEmpty() && knownSubclasses[subclassIdx]!=NULL; subclassIdx++)
    {
      root = XMLNode::parseString(XML, knownSubclasses[subclassIdx]);
    }

  return getInstanceHelper(root, subclassIdx);
}

svlDetectorFeatureExtractor* getInstanceHelper(XMLNode &root, int subclassIdx)
{
  //Punt if the file wasn't XML or didn't have the right root node
  if(root.isEmpty()) {
    SVL_LOG(SVL_LOG_WARNING, "Could not read svlDetectorFeatureExtractor config file.")
    return NULL;
  }
  
  switch(subclassIdx)
    {
    case 0:
      {

	svlPatchDictionary* PD = new svlPatchDictionary();
	PD->read(root);
	return dynamic_cast<svlDetectorFeatureExtractor *>(PD);
      }
    case 1:
      {
	//svlCompositeFeatureExtractor* CFE = new svlCompositeFeatureExtractor();
	//CFE->read(root);
	//return CFE;
	return NULL;
      }
    default:
      SVL_LOG(SVL_LOG_FATAL, "INSANITY!  This shouldn't ever happen.");
      return NULL;
    }

  return NULL;
}

bool svlDetectorFeatureExtractor::write(const string &filename)
{
  return this->write(filename.c_str());
}

bool svlDetectorFeatureExtractor::write(const char *filename)
{
  ofstream ofs(filename);
  bool rval = this->writeOut(ofs);
  ofs.close();
  return rval;
}
