/*****************************************************************************
** STAIR VISION LIBRARY
** Copyright (c) 2007-2009, Stephen Gould
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**     * Redistributions of source code must retain the above copyright
**       notice, this list of conditions and the following disclaimer.
**     * Redistributions in binary form must reproduce the above copyright
**       notice, this list of conditions and the following disclaimer in the
**       documentation and/or other materials provided with the distribution.
**     * Neither the name of the Stanford University nor the
**       names of its contributors may be used to endorse or promote products
**       derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
** ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
******************************************************************************
** FILENAME:    svlKMeansClustering.h
** AUTHOR(S):   Paul Baumstarck <pbaumstarck@stanford.edu>
**
** DESCRIPTION: Class for performing k-means and accelerated k-means on
**			    vectors of svlpoint3d's.
*****************************************************************************/

#pragma once

#include <cmath>
#include <vector>
#include <list>
#include <ctime>

#include "svlDevel.h"
#include "svlVision.h"
#include "svlPlane.h"
#include "../base/svlOptions.h"
#include "../vision/svlPointCloudData.h"
#include "../vision/svlPoint3d.h"
#include "svlCluster3d.h"

using namespace std;

class svlKMeansClustering : public svlCluster3d, public svlOptions {
  public:
	svlKMeansClustering();
	~svlKMeansClustering();
	
	// virtual method implemented
	void cluster(svlPointCloudData &PCD,
				 vector<int> &asgs,
				 vector<vector<int> > &R_ix,
				 vector<svlPlane> &planes,
				 int k);
	// actual k-means methods
	/*
	void do_kmeans(const vector<svlPoint3d> &Pts,
				   const vector<svlPoint3d> &N,
						 vector<int> &asgs,
						 vector<vector<int> > &R_ix,
						 vector<svlCentroid6> &cents,
						 int k);
	void do_kmeans_acc(const vector<svlPoint3d> &Pts,
					   const vector<svlPoint3d> &N,
					      	 vector<svlCentroid6> &cents,
							 vector<int> &asgns,
							 int k);
							 */
	void do_kmeans(const vector<svlPoint3d> &P,
				   const vector<svlPoint3d> &N,
						 vector<int> &ix,
						 vector<vector<int> > &rix,
						 vector<svlPlane> &planes,
						 int k);

	/*void centroidMeanShift(const vector<svlPoint3d> &P,
						   const vector<svlPoint3d> &N,
						   const vector<int> &ix, svlCentroid6*/

	inline void meanShiftCorrect(const vector<svlPoint3d> &P, const vector<svlPoint3d> &N, const vector<int> &ix);
	void setDefaultOptions();

  private:
};

