/*****************************************************************************
** STAIR VISION LIBRARY
** Copyright (c) 2007-2009, Stephen Gould
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**     * Redistributions of source code must retain the above copyright
**       notice, this list of conditions and the following disclaimer.
**     * Redistributions in binary form must reproduce the above copyright
**       notice, this list of conditions and the following disclaimer in the
**       documentation and/or other materials provided with the distribution.
**     * Neither the name of the Stanford University nor the
**       names of its contributors may be used to endorse or promote products
**       derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
** ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
******************************************************************************
** FILENAME:    svlMapping.h
** AUTHOR(S):   Paul Baumstarck <pbaumstarck@stanford.edu>
** DESCRIPTION:
**  Class implementing forward and backward permutation.
**
*****************************************************************************/

#pragma once

#include <functional>
#include <iostream>
#include <vector>
#include <map>
using namespace std;

struct MappingIndex {
	int i;
	MappingIndex() : i(-1) {}
	MappingIndex(int i) : i(i) {}
	~MappingIndex() {}
	friend ostream& operator<<(ostream& os, const MappingIndex& i) {
		os << i.i; return os;
	}
};

class svlMapping {
public:
	map<int, MappingIndex> AtoB, BtoA;
	map<int, MappingIndex>::iterator itA, itB;
	
public:
	// con/destructors
	svlMapping();
	~svlMapping();

	// general functions
	void clear();
	void allocateSize(int size);
	void addAtoB(int a, int b);
	void addBtoA(int b, int a);
	void addAtofromB(int a, int b);
	int getAtoB(int a);
	int getBtoA(int b);

	// iterators
	void setIterABegin();
	void setIterBBegin();
	void incIterA();
	void incIterB();
	pair<int,int> getIterA();
	pair<int,int> getIterB();
	bool iterAEnd();
	bool iterBEnd();

	// weird functions
	vector<int> getRingFromA(int a);
	//vector<int> getRingFromB(int b);
};

