/*****************************************************************************
** STAIR VISION LIBRARY
** Copyright (c) 2007-2009, Stephen Gould
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**     * Redistributions of source code must retain the above copyright
**       notice, this list of conditions and the following disclaimer.
**     * Redistributions in binary form must reproduce the above copyright
**       notice, this list of conditions and the following disclaimer in the
**       documentation and/or other materials provided with the distribution.
**     * Neither the name of the Stanford University nor the
**       names of its contributors may be used to endorse or promote products
**       derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
** ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
******************************************************************************
** FILENAME:    svlRabbaniClustering.h
** AUTHOR(S):   Paul Baumstarck <pbaumstarck@stanford.edu>
**
** DESCRIPTION:
**   Performs the Rabbani greedy region growing clustering technique.
*****************************************************************************/

#pragma once

#include <iostream>
#include <vector>
#include <queue>

#include "cv.h"
#include "cxcore.h"

#include "svlDevel.h"
#include "svlVision.h"
#include "svlCluster3d.h"
#include "svlKdTree.h"
#include "svlSortVectorDI.h"
#include "../vision/svlPoint3d.h"
#include "../vision/svlPointCloudData.h"

#define METHOD_0 1

class svlRabbaniClustering : public svlCluster3d {
public:
	svlRabbaniClustering() {};
	~svlRabbaniClustering() {};

	void cluster(svlPointCloudData &PCD, vector<int> &asgs, vector<vector<int> > &R_ix, svlSortVectorDI &resid);
	void cluster(svlPointCloudData &PCD, vector<vector<int> > &knn, vector<int> &asgs, vector<vector<int> > &R_ix, svlSortVectorDI &resid);
	void cluster(vector<svlPoint3d> &P, vector<svlPoint3d> &N, vector<vector<int> > &knn, vector<int> &asgs, vector<vector<int> > &R_ix, svlSortVectorDI &resid,
		int mode = 0, double r_th_in = 0.05, double th_th_in = 0.95, double th_th2_in = 0.83);
	void cluster(svlPointCloudData &PCD, CvMat *Ix, vector<int> &asgs, vector<vector<int> > &R_ix, svlSortVectorDI &resid, int mode = 0);
private:
};

