/*****************************************************************************
** STAIR VISION LIBRARY
** Copyright (c) 2007-2009, Stephen Gould
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**     * Redistributions of source code must retain the above copyright
**       notice, this list of conditions and the following disclaimer.
**     * Redistributions in binary form must reproduce the above copyright
**       notice, this list of conditions and the following disclaimer in the
**       documentation and/or other materials provided with the distribution.
**     * Neither the name of the Stanford University nor the
**       names of its contributors may be used to endorse or promote products
**       derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
** ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
******************************************************************************
** FILENAME:    svlAbsPatchMaskCreator
** AUTHOR(S):   Olga Russakovsky <olga@cs.stanford.edu>
** DESCRIPTION:
**      This class creates the different mask types supported by the
** svlAbsPatchDefinition class. It should probably just be part of the
** svlPatchDefinition.cpp file, but it was getting too long and messy (and 
** it's still being developed, so I put it here instead.
**
*****************************************************************************/

#pragma once

#include <cassert>
#include <iostream>
#include <vector>

#include "cv.h"
#include "cxcore.h"

#include "xmlParser/xmlParser.h"

#include "svlBase.h"
#include "svlVision.h"


class svlAbsPatchMaskCreator {
 public:
  int _nmasks;

  svlAbsPatchMaskCreator() { _nmasks = 4; }

  int numMasksRequired(int level) {
    return _nmasks;
  }

  int numMasksRequiredDirectional() {
    return 1;
  }

  void createDirectional(int level, CvMat **_masks, CvPoint *_refs, CvPoint &offsetTL, CvPoint &offsetBR) {
    CvMat tmp;

    if (level == 0) {
      float m1a[] = {0, -1, 1, 0};
      tmp = cvMat(2, 2, CV_32FC1, m1a);
      _masks[0] = cvCloneMat(&tmp);
      _refs[0] = cvPoint(0, 1);

      offsetTL.x = max(offsetTL.x, 0);
      offsetTL.y = max(offsetTL.y, 1);
      offsetBR.x = max(offsetBR.x, 1);
      offsetBR.y = max(offsetBR.y, 1);

    } else if (level == 1) {
      float m1b[] = {1, -1};
      tmp = cvMat(1, 2, CV_32FC1, m1b);
      _masks[0] = cvCloneMat(&tmp);
      _refs[0] = cvPoint(0, 0);

      offsetTL.x = max(offsetTL.x, 0);
      offsetTL.y = max(offsetTL.y, 1);
      offsetBR.x = max(offsetBR.x, 1);
      offsetBR.y = max(offsetBR.y, 1);

    } else if (level == 2) {
      float m1c[] = {1, 0, 0, -1};
      tmp = cvMat(2, 2, CV_32FC1, m1c);
      _masks[0] = cvCloneMat(&tmp);
      _refs[0] = cvPoint(0, 0);

      offsetTL.x = max(offsetTL.x, 0);
      offsetTL.y = max(offsetTL.y, 1);
      offsetBR.x = max(offsetBR.x, 1);
      offsetBR.y = max(offsetBR.y, 1);

    } else if (level == 3) {
      float m1d[] = {1, -1};
      tmp = cvMat(2, 1, CV_32FC1, m1d);
      _masks[0] = cvCloneMat(&tmp);
      _refs[0] = cvPoint(0, 0);      

      offsetTL.x = max(offsetTL.x, 0);
      offsetTL.y = max(offsetTL.y, 1);
      offsetBR.x = max(offsetBR.x, 1);
      offsetBR.y = max(offsetBR.y, 1);

    } else {
      assert(false);
    }

      
    /*




    } else if (level == 2) {
      float m2a[] = {0, -1.0/3, -1.0/3, 0, 0, -1.0/3, 1, 0, 0};
      float m2b[] = {0, 0, -1.0/3, 1, 0, -1.0/3, 0, 0, -1.0/3};
      float m2c[] = {1, 0, 0, 0, 0, -1.0/3, 0, -1.0/3, -1.0/3};
      float m2d[] = {0, 1, 0, 0, 0, 0, -1.0/3, -1.0/3, -1.0/3};

      tmp = cvMat(3, 3, CV_32FC1, m2a);
      _masks[0] = cvCloneMat(&tmp);
      
      tmp = cvMat(3, 3, CV_32FC1, m2b);
      _masks[1] = cvCloneMat(&tmp);
      
      tmp = cvMat(3, 3, CV_32FC1, m2c);
      _masks[2] = cvCloneMat(&tmp);
      
      tmp = cvMat(3, 3, CV_32FC1, m2d);
      _masks[3] = cvCloneMat(&tmp);      

      _refs[0] = cvPoint(0, 2);
      _refs[1] = cvPoint(0, 1);
      _refs[2] = cvPoint(0, 0);
      _refs[3] = cvPoint(1, 0);

      offsetTL.x = max(offsetTL.x, 1);
      offsetTL.y = max(offsetTL.y, 2);
      offsetBR.x = max(offsetBR.x, 2);
      offsetBR.y = max(offsetBR.y, 2);

      return _nmasks;
      
    } else if (level == 3) {
      float m3a[] = {0, 0, -1.0/3, -1.0/3,
		     0, 0, 0, -1.0/3,
		     0, 0, 0, 0,
		     1, 0, 0, 0};
      float m3b[] = {0, 0, 0, -1.0/3, 1, 0, 0, -1.0/3, 0, 0, 0, -1.0/3};
      float m3c[] = {1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, -1.0/3, 0, 0, -1.0/3, -1.0/3};
      float m3d[] = {0, 1, 0, 0, 0, 0, 0, 0, 0, -1.0/3, -1.0/3, -1.0/3};

      tmp = cvMat(4, 4, CV_32FC1, m3a);
      _masks[0] = cvCloneMat(&tmp);

      tmp = cvMat(3, 4, CV_32FC1, m3b);
      _masks[1] = cvCloneMat(&tmp);

      tmp = cvMat(4, 4, CV_32FC1, m3c);
      _masks[2] = cvCloneMat(&tmp);

      tmp = cvMat(4, 3, CV_32FC1, m3d);
      _masks[3] = cvCloneMat(&tmp);

      _refs[0] = cvPoint(0, 3);
      _refs[1] = cvPoint(0, 1);
      _refs[2] = cvPoint(0, 0);
      _refs[3] = cvPoint(0, 1);

      offsetTL.x = max(offsetTL.x, 1);
      offsetTL.y = max(offsetTL.y, 3);
      offsetBR.x = max(offsetBR.x, 3);
      offsetBR.y = max(offsetBR.y, 3);

      return _nmasks;
    }
    */
  }



  // assumes everything is allocated and set up properly already
  // returns the number of masks created (same as the numMasksRequired)
  int create(int level, CvMat **_masks, CvPoint *_refs, CvPoint &offsetTL, CvPoint &offsetBR) {
    CvMat tmp;

    if (level == 1) {
      float m1a[] = {0.0f, -1.0f, 1.0f, 0.0f};
      float m1b[] = {1.0f, -1.0f};
      float m1c[] = {1.0f, 0.0f, 0.0f, -1.0f};
      float m1d[] = {1.0f, -1.0f};

      tmp = cvMat(2, 2, CV_32FC1, m1a);
      _masks[0] = cvCloneMat(&tmp);
      
      tmp = cvMat(1, 2, CV_32FC1, m1b);
      _masks[1] = cvCloneMat(&tmp);
      
      tmp = cvMat(2, 2, CV_32FC1, m1c);
      _masks[2] = cvCloneMat(&tmp);
      
      tmp = cvMat(2, 1, CV_32FC1, m1d);
      _masks[3] = cvCloneMat(&tmp);

      _refs[0] = cvPoint(0, 1);
      _refs[1] = cvPoint(0, 0);
      _refs[2] = cvPoint(0, 0);
      _refs[3] = cvPoint(0, 0);

      offsetTL.x = max(offsetTL.x, 0);
      offsetTL.y = max(offsetTL.y, 1);
      offsetBR.x = max(offsetBR.x, 1);
      offsetBR.y = max(offsetBR.y, 1);

      return _nmasks;

    } else if (level == 2) {
      float m2a[] = {0.0f, -1.0f/3.0f, -1.0f/3.0f, 0.0f, 0.0f, -1.0f/3.0f, 1.0f, 0.0f, 0.0f};
      float m2b[] = {0.0f, 0.0f, -1.0f/3.0f, 1.0f, 0.0f, -1.0f/3.0f, 0.0f, 0.0f, -1.0f/3.0f};
      float m2c[] = {1.0f, 0.0f, 0.0f, 0.0f, 0.0f, -1.0f/3.0f, 0.0f, -1.0f/3.0f, -1.0f/3.0f};
      float m2d[] = {0.0f, 1.0f, 0.0f, 0.0f, 0.0f, 0.0f, -1.0f/3.0f, -1.0f/3.0f, -1.0f/3.0f};

      tmp = cvMat(3, 3, CV_32FC1, m2a);
      _masks[0] = cvCloneMat(&tmp);
      
      tmp = cvMat(3, 3, CV_32FC1, m2b);
      _masks[1] = cvCloneMat(&tmp);
      
      tmp = cvMat(3, 3, CV_32FC1, m2c);
      _masks[2] = cvCloneMat(&tmp);
      
      tmp = cvMat(3, 3, CV_32FC1, m2d);
      _masks[3] = cvCloneMat(&tmp);      

      _refs[0] = cvPoint(0, 2);
      _refs[1] = cvPoint(0, 1);
      _refs[2] = cvPoint(0, 0);
      _refs[3] = cvPoint(1, 0);

      offsetTL.x = max(offsetTL.x, 1);
      offsetTL.y = max(offsetTL.y, 2);
      offsetBR.x = max(offsetBR.x, 2);
      offsetBR.y = max(offsetBR.y, 2);

      return _nmasks;
      
    } else if (level == 3) {
      float m3a[] = {0.0f, 0.0f, -1.0f/3.0f, -1.0f/3.0f,
		     0.0f, 0.0f, 0.0f, -1.0f/3.0f,
		     0.0f, 0.0f, 0.0f, 0.0f,
		     1.0f, 0.0f, 0.0f, 0.0f};
      float m3b[] = {0.0f, 0.0f, 0.0f, -1.0f/3.0f, 1.0f, 0.0f, 0.0f, -1.0f/3.0f, 0.0f, 0.0f, 0.0f, -1.0f/3.0f};
      float m3c[] = {1.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, -1.0f/3.0f, 0.0f, 0.0f, -1.0f/3.0f, -1.0f/3.0f};
      float m3d[] = {0.0f, 1.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, 0.0f, -1.0f/3.0f, -1.0f/3.0f, -1.0f/3.0f};

      tmp = cvMat(4, 4, CV_32FC1, m3a);
      _masks[0] = cvCloneMat(&tmp);

      tmp = cvMat(3, 4, CV_32FC1, m3b);
      _masks[1] = cvCloneMat(&tmp);

      tmp = cvMat(4, 4, CV_32FC1, m3c);
      _masks[2] = cvCloneMat(&tmp);

      tmp = cvMat(4, 3, CV_32FC1, m3d);
      _masks[3] = cvCloneMat(&tmp);

      _refs[0] = cvPoint(0, 3);
      _refs[1] = cvPoint(0, 1);
      _refs[2] = cvPoint(0, 0);
      _refs[3] = cvPoint(0, 1);

      offsetTL.x = max(offsetTL.x, 1);
      offsetTL.y = max(offsetTL.y, 3);
      offsetBR.x = max(offsetBR.x, 3);
      offsetBR.y = max(offsetBR.y, 3);

      return _nmasks;

    } else if (level == 4) {
      float m4a[] = {0, 0, -0.125f, -0.125f, -0.125f,
		     0, 0, -0.125f, -0.125f, -0.125f,
		     0, 0,      0, -0.125f, -0.125f,
		     0, 0,      0,      0,      0,
		     1, 0,      0,      0,      0};

      float m4b[] = {0, 0, 0, -0.1f, -0.1f,
		     0, 0, 0, -0.1f, -0.1f,
		     1, 0, 0, -0.1f, -0.1f,
		     0, 0, 0, -0.1f, -0.1f,
		     0, 0, 0, -0.1f, -0.1f};

      float m4c[] = {1, 0,     0,     0,     0,
		     0, 0,     0,     0,     0,
		     0, 0,     0,  -0.125f, -0.125f,
		     0, 0, -0.125f, -0.125f, -0.125f,
		     0, 0, -0.125f, -0.125f, -0.125f};

      float m4d[] = {   0,    0,    1,    0,    0,
			0,    0,    0,    0,    0,
			0,    0,    0,    0,    0,
		     -0.1f, -0.1f, -0.1f, -0.1f, -0.1f, 
		     -0.1f, -0.1f, -0.1f, -0.1f, -0.1f};

      tmp = cvMat(5, 5, CV_32FC1, m4a);
      _masks[0] = cvCloneMat(&tmp);

      tmp = cvMat(5, 5, CV_32FC1, m4b);
      _masks[1] = cvCloneMat(&tmp);

      tmp = cvMat(5, 5, CV_32FC1, m4c);
      _masks[2] = cvCloneMat(&tmp);

      tmp = cvMat(5, 5, CV_32FC1, m4d);
      _masks[3] = cvCloneMat(&tmp);

      _refs[0] = cvPoint(0, 4);
      _refs[1] = cvPoint(0, 2);
      _refs[2] = cvPoint(0, 0);
      _refs[3] = cvPoint(0, 2);

      offsetTL.x = max(offsetTL.x, 2);
      offsetTL.y = max(offsetTL.y, 4);
      offsetBR.x = max(offsetBR.x, 4);
      offsetBR.y = max(offsetBR.y, 4);

      return _nmasks;
    } else {
      SVL_ASSERT_MSG(false, "this level is not defined");
      return 0;
    }
  }

};


