/*****************************************************************************
** STAIR VISION LIBRARY
** Copyright (c) 2007-2009, Stephen Gould
** All rights reserved.
**
** Redistribution and use in source and binary forms, with or without
** modification, are permitted provided that the following conditions are met:
**     * Redistributions of source code must retain the above copyright
**       notice, this list of conditions and the following disclaimer.
**     * Redistributions in binary form must reproduce the above copyright
**       notice, this list of conditions and the following disclaimer in the
**       documentation and/or other materials provided with the distribution.
**     * Neither the name of the Stanford University nor the
**       names of its contributors may be used to endorse or promote products
**       derived from this software without specific prior written permission.
**
** THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS ``AS IS'' AND ANY
** EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
** WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
** DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
** DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
** (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
** LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
** ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
** (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
** SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
**
******************************************************************************
** FILENAME:    svlImageSequence.h
** AUTHOR(S):   Stephen Gould <sgould@stanford.edu>
**              Ian Goodfellow <ian3@cs.stanford.edu>
** DESCRIPTION:
**  Defines an image sequence. The idea is that this can be used as a
**  surrogate video.
**  Defines some operations on image sequences.
**
*****************************************************************************/

#pragma once

#include <vector>
#include <iostream>
#include <string>

#include "svlBase.h"

using namespace std;

class svlImageSequence {
 public:
    svlImageSequence();
    svlImageSequence(const svlImageSequence& s);
    virtual ~svlImageSequence();

    void load(const char *filename); //warning-- this adds the contents of the file to the sequence, rather than replacing the sequence's contents with the file's contents. can result in bugs if the file is for a different directory
    void save(const char *filename) const;
    void dir(const char *directory, const char *extension = ".jpg");

    void clear() { _imageNames.clear(); }
    bool empty() const { return _imageNames.empty(); }
    unsigned size() const { return (unsigned)_imageNames.size(); }
    void push_back(const string& s) { _imageNames.push_back(s); }
    void pop_back() { _imageNames.pop_back(); }
    string & back() { return _imageNames.back(); }
    string back() const { return _imageNames.back(); }

    // Returns the i-th image. Calling function should not free the memory.
    const IplImage *image(unsigned index);

    // Returns the directory name.
    const string& directory() const { return _directoryName; }
    void setDirectory(string directoryName) { _directoryName = directoryName; }

    // Returns the i-th's image name.
    const string operator[](unsigned i) const { return _imageNames[i]; }
    string& operator[](unsigned i) { return _imageNames[i]; }

 protected:
    string _directoryName;
    vector<string> _imageNames;
    IplImage *_imageBuffer;
};

//Operations on image sequences
//(Feel free to move or rename these if the location offends you, but be aware that the new command line system for the object detector apps depends on them)

//Checks that a sequence contains a non-zero number of images, all with the same
//file extension
bool hasHomogeneousExtensions(const svlImageSequence & s);

//Returns the image extension used by the sequence, if it has
//homogenous extensions
//Throws an assert if sequence is empty
//If sequence is non-empty but not homogenous then this extension will not
//be correct for all images... obviously
string getExtension(const svlImageSequence & s);
